/*
 * Copyright (c) 2008 appspy.org, Contributors.
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
 * Unless required by applicable law or agreed to in writing, software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions 
 * and limitations under the License. */

package org.appspy.viewer.client;

import org.appspy.viewer.client.dashboards.action.OpenDashboardAction;
import org.appspy.viewer.client.form.DashboardForm;
import org.appspy.viewer.client.form.ReportForm;
import org.appspy.viewer.client.paramchooser.ParamUtils;
import org.appspy.viewer.client.reports.actions.OpenReportAction;

import com.google.gwt.user.client.rpc.AsyncCallback;
import com.gwtext.client.widgets.MessageBox;

/**
 * @author Olivier HEDIN / olivier@appspy.org
 */
public class ReportingFunctions {
	protected ViewerModule mViewerModule = null;
	
	public ReportingFunctions(ViewerModule viewerModule)
	{
		mViewerModule = viewerModule;
	}

	public void openReport(String reportName, final boolean askParams, final String params)
	{
		mViewerModule.getViewerModuleService().getReportByName(reportName, new AsyncCallback<ReportForm>(){

			public void onFailure(Throwable caught) {
				MessageBox.alert("Error", caught.getLocalizedMessage());
			}

			public void onSuccess(ReportForm result) {
				ParamUtils.queryStringToParams(params, result.getParams());
				
				OpenReportAction action = new OpenReportAction(mViewerModule, result, askParams);
				action.execute();
			}});
		
	}

	public void openDashboard(String dashboardName, final boolean askParams, final String params)
	{
		mViewerModule.getViewerModuleService().getDashboardByName(dashboardName, new AsyncCallback<DashboardForm>(){

			public void onFailure(Throwable caught) {
				MessageBox.alert("Error", caught.getLocalizedMessage());
			}

			public void onSuccess(DashboardForm result) {
				ParamUtils.queryStringToParams(params, result.getParams());
				
				OpenDashboardAction action = new OpenDashboardAction(mViewerModule, result, askParams);
				action.execute();
			}});
		
	}
}
