/*
 * Copyright (c) 2008 appspy.org, Contributors.
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
 * Unless required by applicable law or agreed to in writing, software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions 
 * and limitations under the License. */

package org.appspy.viewer.client;

import org.appspy.viewer.client.dashboards.DashboardsMenuTreePanel;
import org.appspy.viewer.client.reports.ReportsMenuTreePanel;
import org.appspy.viewer.client.rpc.ViewerModuleService;
import org.appspy.viewer.client.rpc.ViewerModuleServiceAsync;

import com.google.gwt.core.client.EntryPoint;
import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.rpc.ServiceDefTarget;
import com.gwtext.client.core.Margins;
import com.gwtext.client.core.RegionPosition;
import com.gwtext.client.widgets.Panel;
import com.gwtext.client.widgets.TabPanel;
import com.gwtext.client.widgets.Viewport;
import com.gwtext.client.widgets.layout.AccordionLayout;
import com.gwtext.client.widgets.layout.BorderLayout;
import com.gwtext.client.widgets.layout.BorderLayoutData;
import com.gwtext.client.widgets.layout.FitLayout;

/**
 * @author Olivier HEDIN / olivier@appspy.org
 */
public class ViewerModule implements EntryPoint {

	protected TabPanel mTabPanel = null;
	protected ViewerModuleServiceAsync mViewerModuleService = null;

	public void onModuleLoad() {

        final Panel mainPanel = new Panel();
        mainPanel.setLayout(new BorderLayout());


        //setup the west regions layout properties
        BorderLayoutData westLayoutData = new BorderLayoutData(RegionPosition.WEST);
        westLayoutData.setMargins(new Margins(5, 5, 0, 5));
        westLayoutData.setCMargins(new Margins(5, 5, 5, 5));
        westLayoutData.setMinSize(155);
        westLayoutData.setMaxSize(350);
        westLayoutData.setSplit(true);

		Panel menuPanel = new Panel();
		menuPanel.setTitle("Menu");
		menuPanel.setCollapsible(true);
		menuPanel.setWidth(200);
		final AccordionLayout accordion = new AccordionLayout();
		accordion.setAnimate(true);
		menuPanel.setLayout(accordion);

		AbstractTreePanel dashboardsPanel = new DashboardsMenuTreePanel(this);
		dashboardsPanel.setViewerModule(this);
		dashboardsPanel.setTitle("Dashboards");
		dashboardsPanel.setAutoScroll(true);
		dashboardsPanel.setBorder(false);
		dashboardsPanel.setIconCls("reports-menu-icon");
		menuPanel.add(dashboardsPanel);
		
		AbstractTreePanel reportsPanel = new ReportsMenuTreePanel(this);
		reportsPanel.setTitle("Reports");
		reportsPanel.setAutoScroll(true);
		reportsPanel.setBorder(false);
		reportsPanel.setIconCls("reports-menu-icon");
		menuPanel.add(reportsPanel);

        mainPanel.add(menuPanel, westLayoutData);

        BorderLayoutData centerLayoutData = new BorderLayoutData(RegionPosition.CENTER);
        centerLayoutData.setMargins(new Margins(5, 0, 5, 5));

		final Panel welcomeTab = new Panel();
		welcomeTab.setTitle("Welcome");
		welcomeTab.setClosable(true);
		welcomeTab.setLayout(new FitLayout());
		welcomeTab.setAutoWidth(true);
		welcomeTab.setAutoScroll(true);
		Panel p = new Panel();
		p.setLayout(new FitLayout());
		String html = "<h2>Welcome to the AppSpy Viewer !</h2>";
		html = html + "<br/>";
		html = html + "<p>Appspy is released under the <a target='_blank' href='http://www.apache.org/licenses/LICENSE-2.0.html'>Apache License, Version 2.0</a></p>";
		html = html + "<br/>";
		html = html + "<p>Copyright (c) 2008 <a target='_blank' href='http://www.appspy.org'>appspy.org</a></p>";
		p.setHtml(html);
		p.setBorder(false);
		p.setAutoWidth(true);
		p.setAutoScroll(true);
		welcomeTab.add(p);
		
        final Panel centerPanelWrappper = new Panel();
        centerPanelWrappper.setLayout(new FitLayout());
        centerPanelWrappper.setBorder(false);
        centerPanelWrappper.setBodyBorder(false);
        centerPanelWrappper.setFrame(true);

        mTabPanel = new TabPanel();
        mTabPanel.setBodyBorder(false);
        mTabPanel.setEnableTabScroll(true);
        mTabPanel.setAutoScroll(true);

        mTabPanel.setLayoutOnTabChange(true);
        mTabPanel.setTitle("Main Content");

		mTabPanel.add(welcomeTab);
		mTabPanel.setActiveTab(0);

        mainPanel.add(mTabPanel, centerLayoutData);

        @SuppressWarnings("unused")
		Viewport viewport = new Viewport(mainPanel);

        mainPanel.doLayout();
        mainPanel.show();
        
        initJNSI(new ReportingFunctions(this));
	}
	
	/**
	 * @return the tabPanel
	 */
	public TabPanel getTabPanel() {
		return mTabPanel;
	}

	public ViewerModuleServiceAsync getViewerModuleService() {
		if (mViewerModuleService == null)
		{
			mViewerModuleService = (ViewerModuleServiceAsync) GWT
			.create(ViewerModuleService.class);
			ServiceDefTarget endpoint = (ServiceDefTarget) mViewerModuleService;
			String moduleRelativeURL = GWT.getModuleBaseURL()
			+ "viewerModuleService";
			endpoint.setServiceEntryPoint(moduleRelativeURL);
		}

		return mViewerModuleService;
	}
	
	public native void initJNSI(ReportingFunctions obj)/*-{
	
	$wnd.openReport = function(reportName, askParams, params) {obj.@org.appspy.viewer.client.ReportingFunctions::openReport(Ljava/lang/String;ZLjava/lang/String;)(reportName, askParams, params);};
	$wnd.openDashboard = function(dashboardName, askParams, params) {obj.@org.appspy.viewer.client.ReportingFunctions::openDashboard(Ljava/lang/String;ZLjava/lang/String;)(dashboardName, askParams, params);};
	
	}-*/;

}
