/*
 * Copyright (c) 2008 appspy.org, Contributors.
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
 * Unless required by applicable law or agreed to in writing, software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions 
 * and limitations under the License. */

package org.appspy.viewer.client.action;

import org.appspy.viewer.client.ViewerModule;
import org.appspy.viewer.client.ViewerPanel;

import com.gwtext.client.widgets.Panel;
import com.gwtext.client.widgets.layout.FitLayout;

/**
 * @author Olivier HEDIN / olivier@appspy.org
 */
public class OpenAdminPanelAction extends AbstractAction {
	
	protected ViewerPanel mAdminPanel = null;
	protected boolean mIsNewTab = false;
	
	public OpenAdminPanelAction(ViewerModule adminModule, ViewerPanel formPanel, boolean isNewTab)
	{
		super(adminModule);
		mAdminPanel = formPanel;
		mIsNewTab = isNewTab;
	}

	/* (non-Javadoc)
	 * @see org.appspy.admin.client.action.AbstractAction#execute()
	 */
	@Override
	public void execute() {
		
		Panel formPanel = (Panel) mAdminPanel;
		
		if (mIsNewTab)
		{
			Panel p = null;
			
			p = new Panel();
			p.setLayout(new FitLayout());
			p.setAutoScroll(true);
			p.setClosable(true);
			p.setTitle(formPanel.getTitle());
//			p.setIconCls("tab-icon");
			
			mAdminPanel.setViewerModule(mViewerModule);
			p.add(formPanel);
			
			// Open in new tab
			mViewerModule.getTabPanel().add(p);
			mViewerModule.getTabPanel().activate(p.getId());
		}
		else
		{
			Panel p = mViewerModule.getTabPanel().getActiveTab();
			
			p.setTitle(formPanel.getTitle());
			
			mAdminPanel.setViewerModule(mViewerModule);
			
			p.clear();
			p.add(formPanel);
			
			p.doLayout();
		}
	}

}
