/*
 * Copyright (c) 2008 appspy.org, Contributors.
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
 * Unless required by applicable law or agreed to in writing, software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions 
 * and limitations under the License. */

package org.appspy.viewer.client.dashboards.action;

import org.appspy.viewer.client.ViewerModule;
import org.appspy.viewer.client.action.AbstractAction;
import org.appspy.viewer.client.dashboards.DashboardPanel;
import org.appspy.viewer.client.form.DashboardForm;
import org.appspy.viewer.client.paramchooser.ParamWindow;

import com.gwtext.client.widgets.Panel;
import com.gwtext.client.widgets.TabPanel;
import com.gwtext.client.widgets.layout.FitLayout;

/**
 * @author Olivier HEDIN / olivier@appspy.org
 */
public class OpenDashboardAction extends AbstractAction {

	protected DashboardForm mDashboard = null;
	protected boolean mAskParams = true;

	/**
	 * @param viewerModule
	 */
	public OpenDashboardAction(ViewerModule viewerModule, DashboardForm dashboard, boolean askParams) {
		super(viewerModule);

		mDashboard = dashboard;
		mAskParams = askParams;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.appspy.admin.client.action.AbstractAction#execute()
	 */
	@Override
	public void execute() {
		final ParamWindow paramWindow = new ParamWindow(mViewerModule, mDashboard.getParams(), new AbstractAction(mViewerModule){

			@Override
			public void execute() {
				TabPanel tabPanel = mViewerModule.getTabPanel();

				Panel p = new Panel();
				DashboardPanel c = new DashboardPanel(mViewerModule, mDashboard);

				p.setLayout(new FitLayout());
				p.setAutoScroll(true);
				p.setClosable(true);
				p.setTitle(mDashboard.getName());
				p.setIconCls("reports-menu-icon");

				p.add(c);

				try {
					tabPanel.add(p);
					tabPanel.activate(p.getId());
				} catch (Exception ex) {
					ex.printStackTrace();
				}
			}});

		paramWindow.setModal(true);
		if (mAskParams)
		{
			paramWindow.show();
		}
		else
		{
			paramWindow.executeAction();
			paramWindow.destroy();
		}
	}

}
