/*
 * Copyright (c) 2008 appspy.org, Contributors.
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
 * Unless required by applicable law or agreed to in writing, software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions 
 * and limitations under the License. */

package org.appspy.viewer.client.paramchooser;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import org.appspy.viewer.client.ViewerModule;
import org.appspy.viewer.client.form.ReportParamForm;
import org.appspy.viewer.client.util.Calendar;
import org.appspy.viewer.client.util.GregorianCalendar;

import com.google.gwt.core.client.GWT;
import com.gwtext.client.util.DateUtil;
import com.gwtext.client.widgets.Panel;
import com.gwtext.client.widgets.form.DateField;
import com.gwtext.client.widgets.form.Field;
import com.gwtext.client.widgets.form.event.TextFieldListenerAdapter;

/**
 * @author Olivier HEDIN / olivier@appspy.org
 */
public class DateChooser extends DateField implements ParamChooser {
	
	protected ReportParamForm mParam = null;
	protected ViewerModule mViewerModule = null;
	protected Panel mPropertyPanel = null;
	protected Map<String, String> mPropertyGridMap = null;
	
	protected static String sDateFormat = "Y/m/d";
	
	public DateChooser(ViewerModule viewerModule, ReportParamForm param, Panel propertyPanel)
	{
		super("", sDateFormat);
		mParam = param;
		mViewerModule = viewerModule;
		mPropertyPanel = propertyPanel;
		
		addListener(new TextFieldListenerAdapter(){

			@Override
			public void onValid(Field field) {
				Date newDate = (Date) ((DateField)field).getValue();
				mParam.setDefault(DateUtil.format(newDate, sDateFormat));
			}});
	}

	/**
	 * @return the param
	 */
	public ReportParamForm getParam() {
		return mParam;
	}

	/**
	 * @param param the param to set
	 */
	public void setParam(ReportParamForm param) {
		mParam = param;
	}

	/* (non-Javadoc)
	 * @see org.appspy.viewer.client.paramchooser.ParamChooser#getParamDisplayValue()
	 */
	public String getParamDisplayValue() {
		return getValueAsString();
	}

	/* (non-Javadoc)
	 * @see org.appspy.viewer.client.paramchooser.ParamChooser#getParamValue()
	 */
	public String getParamValue() {
		return getValueAsString();
	}

	/* (non-Javadoc)
	 * @see org.appspy.viewer.client.paramchooser.ParamChooser#updateValue()
	 */
	public void updateValue() {
		
		GWT.log("dans updateValue : " + mParam.getDefault(), null);
			if (mParam.getDefault().matches("[0-9]+/[0-9]+/[0-9]+"))
			{
				GWT.log("matches !", null);
				DateUtil.parseDate(mParam.getDefault(), sDateFormat);
			}
			else
			{
				GregorianCalendar cal = new GregorianCalendar();
				cal.setTime(new Date());
				
				HashMap<String, Integer> fieldMap = new HashMap<String, Integer>();
				fieldMap.put("DATE", Calendar.DATE);
				fieldMap.put("DAY_OF_MONTH", Calendar.DAY_OF_MONTH);
				fieldMap.put("DAY_OF_WEEK", Calendar.DAY_OF_WEEK);
				fieldMap.put("DAY_OF_WEEK_IN_MONTH", Calendar.DAY_OF_WEEK_IN_MONTH);
				fieldMap.put("DAY_OF_YEAR", Calendar.DAY_OF_YEAR);
				fieldMap.put("HOUR", Calendar.HOUR);
				fieldMap.put("HOUR_OF_DAY", Calendar.HOUR_OF_DAY);
				fieldMap.put("MILLISECOND", Calendar.MILLISECOND);
				fieldMap.put("MINUTE", Calendar.MINUTE);
				fieldMap.put("MONTH", Calendar.MONTH);
				fieldMap.put("SECOND", Calendar.SECOND);
				fieldMap.put("WEEK_OF_MONTH", Calendar.WEEK_OF_MONTH);
				fieldMap.put("WEEK_OF_YEAR", Calendar.WEEK_OF_YEAR);
				fieldMap.put("YEAR", Calendar.YEAR);
				
				try
				{
					String[] lines = mParam.getDefault().split(";");
					for (String line : lines) {
						String[] s = line.split(" ");
						String cmd = s[0];
						String field = s[1].toUpperCase();
						String val = s[2];
						
						if ("set".equalsIgnoreCase(cmd))
						{
							if (fieldMap.containsKey(field))
							{
								cal.set(fieldMap.get(field), val);
							}
						}
						else if ("add".equalsIgnoreCase(cmd))
						{
							if (fieldMap.containsKey(field))
							{
								cal.add(fieldMap.get(field), Integer.parseInt(val));
							}
						}
					}
				}
				catch (Exception ex)
				{
					GWT.log(ex.getMessage(), ex);
				}

				String d = DateUtil.format(cal.getTime(), sDateFormat);
				mParam.setDefault(d);
				mPropertyGridMap.put(mParam.getName(), d);
			}
			
			setValue(mParam.getDefault());
	}
}
