/*
 * Copyright (c) 2008 appspy.org, Contributors.
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
 * Unless required by applicable law or agreed to in writing, software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions 
 * and limitations under the License. */

package org.appspy.viewer.client.paramchooser;

import java.util.Date;
import java.util.HashMap;

import org.appspy.viewer.client.ViewerModule;
import org.appspy.viewer.client.form.ReportParamForm;
import org.appspy.viewer.client.util.Calendar;
import org.appspy.viewer.client.util.GregorianCalendar;

import com.google.gwt.core.client.GWT;
import com.gwtext.client.core.EventObject;
import com.gwtext.client.util.DateUtil;
import com.gwtext.client.widgets.Button;
import com.gwtext.client.widgets.Component;
import com.gwtext.client.widgets.DatePicker;
import com.gwtext.client.widgets.Panel;
import com.gwtext.client.widgets.Window;
import com.gwtext.client.widgets.event.ButtonListenerAdapter;
import com.gwtext.client.widgets.event.DatePickerListenerAdapter;
import com.gwtext.client.widgets.form.Field;
import com.gwtext.client.widgets.form.FormPanel;
import com.gwtext.client.widgets.form.TimeField;
import com.gwtext.client.widgets.form.event.FieldListenerAdapter;
import com.gwtext.client.widgets.layout.AnchorLayoutData;
import com.gwtext.client.widgets.layout.FitLayout;

/**
 * @author Olivier HEDIN / olivier@appspy.org
 */
public class DateTimeChooser extends TriggerFieldParamChooser {
	
	public static String sDateFormat = "Y/m/d";
	public static String sTimeFormat = "H:i:s";
	public static String sDateTimeFormat = sDateFormat + " " + sTimeFormat;

	protected String mParamValue = null;
	protected String mParamDisplayValue = null;
	
	public DateTimeChooser(ViewerModule viewerModule, ReportParamForm param, Panel propertyPanel)
	{
		super(viewerModule, param, propertyPanel);
	}
	
	/* (non-Javadoc)
	 * @see com.gwtext.client.widgets.form.TriggerField#onTriggerClick(com.gwtext.client.core.EventObject)
	 */
	@Override
	protected void onTriggerClick(EventObject event) {
		try
		{
			final Window w = new Window(mParam.getPrompt());
			w.setFrame(true);
			w.setModal(true);
			w.setLayout(new FitLayout());
			w.setSize(200, 300);
			
			w.add(getChooserComponent());
			
			w.addButton(new Button("OK", new ButtonListenerAdapter(){

				@Override
				public void onClick(Button button, EventObject e) {
					try
					{
						mParam.setDefault(mParamValue);
						setValue(mParamDisplayValue);
						
						w.destroy();
					}
					catch (Exception ex)
					{
						GWT.log(ex.getMessage(), ex);
					}
				}}));
			
			w.addButton(new Button("Cancel", new ButtonListenerAdapter(){

				@Override
				public void onClick(Button button, EventObject e) {
					w.destroy();
				}}));
			
			w.show();
		}
		catch (Exception ex)
		{
			GWT.log(ex.getMessage(), ex);
		}
	}
	
	/* (non-Javadoc)
	 * @see org.appspy.admin.client.paramchooser.ParamChooser#getChooserComponent()
	 */
	public Component getChooserComponent() {
		
		FormPanel formPanel = new FormPanel();
		
//		formPanel.setTitle(mParam.getPrompt());
		formPanel.setAutoScroll(true);
		formPanel.setBorder(false);
		formPanel.setHideLabels(true);
		
		Date date = DateUtil.parseDate(mParam.getDefault(), sDateTimeFormat);
		
		DatePicker datePicker = new DatePicker();
		datePicker.setValue(date);
		formPanel.add(datePicker);
		
		TimeField timeField = new TimeField("time", sTimeFormat);
		timeField.setFormat(sTimeFormat);
		timeField.setValue(DateUtil.format(date, sTimeFormat));
		formPanel.add(timeField, new AnchorLayoutData("100%"));
		
		datePicker.addListener(new DatePickerListenerAdapter(){
			
			@Override
			public void onSelect(DatePicker dataPicker, Date date) {
				try
				{
					GregorianCalendar newCal = new GregorianCalendar();
					newCal.setTime(date);
					Date oldDate = DateUtil.parseDate(mParamValue, sDateTimeFormat);
					GregorianCalendar cal = new GregorianCalendar();
					cal.setTime(oldDate);
					cal.set(Calendar.YEAR, newCal.get(Calendar.YEAR));
					cal.set(Calendar.MONTH, newCal.get(Calendar.MONTH));
					cal.set(Calendar.DATE, newCal.get(Calendar.DATE));
					mParamValue = DateUtil.format(cal.getTime(), sDateTimeFormat);
					mParamDisplayValue = mParamValue;
				}
				catch (Exception ex)
				{
					GWT.log(ex.getMessage(), ex);
				}
			}});
		
		timeField.addListener(new FieldListenerAdapter(){

			@Override
			public void onValid(Field field) {
				try
				{
					GregorianCalendar newCal = new GregorianCalendar();
					newCal.setTime(DateUtil.parseDate(field.getValueAsString(), sTimeFormat));
					Date oldDate = DateUtil.parseDate(mParamValue, sDateTimeFormat);
					GregorianCalendar cal = new GregorianCalendar();
					cal.setTime(oldDate);
					cal.set(Calendar.HOUR, newCal.get(Calendar.HOUR_OF_DAY));
					cal.set(Calendar.MINUTE, newCal.get(Calendar.MINUTE));
					cal.set(Calendar.SECOND, newCal.get(Calendar.SECOND));
					mParamValue = DateUtil.format(cal.getTime(), sDateTimeFormat);
					mParamDisplayValue = mParamValue;
				}
				catch (Exception ex)
				{
					GWT.log(ex.getMessage(), ex);
				}
			}});
		
		return formPanel;
//					mParamDisplayValue = app.getName();
//					mParamValue = app.getId().toString();
	}

	public void updateValue() {
		if (mParam.getDefault().matches("[0-9]+/[0-9]+/[0-9]+ [0-9]+:[0-9]+:[0-9]+"))
		{
			mParamValue = mParam.getDefault();
			mParamDisplayValue = mParamValue;
		}
		else
		{
			GregorianCalendar cal = new GregorianCalendar();
			cal.setTime(new Date());
			
			HashMap<String, Integer> fieldMap = new HashMap<String, Integer>();
			fieldMap.put("DATE", Calendar.DATE);
			fieldMap.put("DAY_OF_MONTH", Calendar.DAY_OF_MONTH);
			fieldMap.put("DAY_OF_WEEK", Calendar.DAY_OF_WEEK);
			fieldMap.put("DAY_OF_WEEK_IN_MONTH", Calendar.DAY_OF_WEEK_IN_MONTH);
			fieldMap.put("DAY_OF_YEAR", Calendar.DAY_OF_YEAR);
			fieldMap.put("HOUR", Calendar.HOUR);
			fieldMap.put("HOUR_OF_DAY", Calendar.HOUR_OF_DAY);
			fieldMap.put("MILLISECOND", Calendar.MILLISECOND);
			fieldMap.put("MINUTE", Calendar.MINUTE);
			fieldMap.put("MONTH", Calendar.MONTH);
			fieldMap.put("SECOND", Calendar.SECOND);
			fieldMap.put("WEEK_OF_MONTH", Calendar.WEEK_OF_MONTH);
			fieldMap.put("WEEK_OF_YEAR", Calendar.WEEK_OF_YEAR);
			fieldMap.put("YEAR", Calendar.YEAR);
			
			try
			{
				String[] lines = mParam.getDefault().split(";");
				for (String line : lines) {
					String[] s = line.split(" ");
					String cmd = s[0];
					String field = s[1].toUpperCase();
					String val = s[2];
					
					if ("set".equalsIgnoreCase(cmd))
					{
						if (fieldMap.containsKey(field))
						{
							cal.set(fieldMap.get(field), val);
						}
					}
					else if ("add".equalsIgnoreCase(cmd))
					{
						if (fieldMap.containsKey(field))
						{
							cal.add(fieldMap.get(field), Integer.parseInt(val));
						}
					}
				}
			}
			catch (Exception ex)
			{
				GWT.log(ex.getMessage(), ex);
			}

			String d = DateUtil.format(cal.getTime(), sDateTimeFormat);
			mParam.setDefault(d);
			setValue(d);
			mParamValue = d;
			mParamDisplayValue = mParamValue;
		}
		setValue(mParam.getDefault());
	}

	/**
	 * @return the paramValue
	 */
	public String getParamValue() {
		return mParamValue;
	}

	/**
	 * @param paramValue the paramValue to set
	 */
	public void setParamValue(String paramValue) {
		mParamValue = paramValue;
	}

	/**
	 * @return the paramDisplayValue
	 */
	public String getParamDisplayValue() {
		return mParamDisplayValue;
	}

	/**
	 * @param paramDisplayValue the paramDisplayValue to set
	 */
	public void setParamDisplayValue(String paramDisplayValue) {
		mParamDisplayValue = paramDisplayValue;
	}
	
}
