/*
 * Copyright (c) 2008 appspy.org, Contributors.
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
 * Unless required by applicable law or agreed to in writing, software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions 
 * and limitations under the License. */

package org.appspy.viewer.client.paramchooser;

import java.util.Collection;

import org.appspy.viewer.client.ViewerModule;
import org.appspy.viewer.client.form.HostForm;
import org.appspy.viewer.client.form.ReportParamForm;

import com.google.gwt.user.client.rpc.AsyncCallback;
import com.gwtext.client.data.FieldDef;
import com.gwtext.client.data.IntegerFieldDef;
import com.gwtext.client.data.Record;
import com.gwtext.client.data.RecordDef;
import com.gwtext.client.data.Store;
import com.gwtext.client.data.StringFieldDef;
import com.gwtext.client.widgets.Panel;
import com.gwtext.client.widgets.form.ComboBox;
import com.gwtext.client.widgets.form.event.ComboBoxListenerAdapter;

/**
 * @author Olivier HEDIN / olivier@appspy.org
 */
public class HostChooser extends ComboBox implements ParamChooser {
	
	protected ReportParamForm mParam = null;
	protected ViewerModule mViewerModule = null;
	protected Panel mPropertyPanel = null;
	
	protected static RecordDef sRecordDef = new RecordDef(new FieldDef[]{
			new IntegerFieldDef("id"),
			new StringFieldDef("name")
	});
	protected Store mStore = new Store(sRecordDef);
	
	public HostChooser(ViewerModule viewerModule, ReportParamForm param, Panel propertyPanel)
	{
		super();
		mParam = param;
		mViewerModule = viewerModule;
		mPropertyPanel = propertyPanel;
		setReadOnly(true);
		
		setLazyRender(true);
		setStore(mStore);
		setValueField("id");
		setDisplayField("name");
        setMode(ComboBox.LOCAL);
        setForceSelection(true);
        setDisableKeyFilter(true);
        setTriggerAction(ComboBox.ALL);
        
        addListener(new ComboBoxListenerAdapter(){
            @Override
            public void onSelect(ComboBox comboBox, Record record, int index)
            {
	            mParam.setDefault(record.getAsString("id"));
            }});

		mViewerModule.getViewerModuleService().getHosts(new AsyncCallback<Collection<HostForm>>(){

			public void onFailure(Throwable caught) {
			}

			public void onSuccess(Collection<HostForm> result) {
				mStore.removeAll();
				mStore.commitChanges();
				for (HostForm host : result)
                {
	                Record record = sRecordDef.createRecord(new Object[]{
	                		host.getId(),
	                		host.getAlias()
	                });
	                mStore.add(record);
                }
				mStore.commitChanges();
				setValue(mParam.getDefault());
			}});
	}
	
	public void updateValue() {
	}

	/* (non-Javadoc)
     * @see org.appspy.viewer.client.paramchooser.ParamChooser#getParam()
     */
    public ReportParamForm getParam()
    {
	    return mParam;
    }

	/* (non-Javadoc)
     * @see org.appspy.viewer.client.paramchooser.ParamChooser#getParamDisplayValue()
     */
    public String getParamDisplayValue()
    {
	    return getValue();
    }

	/* (non-Javadoc)
     * @see org.appspy.viewer.client.paramchooser.ParamChooser#getParamValue()
     */
    public String getParamValue()
    {
	    return getValue();
    }

	/* (non-Javadoc)
     * @see org.appspy.viewer.client.paramchooser.ParamChooser#setParam(org.appspy.viewer.client.form.ReportParamForm)
     */
    public void setParam(ReportParamForm param)
    {
	    mParam = param;
    }

}
