/*
 * Copyright (c) 2008 appspy.org, Contributors.
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
 * Unless required by applicable law or agreed to in writing, software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions 
 * and limitations under the License. */

package org.appspy.viewer.client.paramchooser;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.TreeMap;

import org.appspy.viewer.client.ViewerModule;
import org.appspy.viewer.client.form.ReportParamForm;

import com.gwtext.client.widgets.Panel;
import com.gwtext.client.widgets.form.Field;
import com.gwtext.client.widgets.form.FormPanel;
import com.gwtext.client.widgets.layout.AnchorLayoutData;

/**
 * @author Olivier HEDIN / olivier@appspy.org
 */
public class ParamPropertyGridPanelFactory {
	
	protected static HashMap<String, String> sLastInputValues = new HashMap<String, String>();
	
	public static Panel getPropertyGridPanel(ViewerModule viewerModule, Collection<ReportParamForm> params)
	{
		final FormPanel propertyPanel = new FormPanel();
		propertyPanel.setAutoScroll(true);
		propertyPanel.setFrame(true);

		final TreeMap<String, ParamChooser> paramChoosers = new TreeMap<String, ParamChooser>();
		for (ReportParamForm param : params) {
			if (param.getAsk())
			{
				if (param.getDefault() == null || "".equals(param.getDefault()))
				{
					if (sLastInputValues.containsKey(param.getName()))
					{
						param.setDefault(sLastInputValues.get(param.getName()));
					}
				}
				ParamChooser paramChooser = ParamChooserFactory.getParamChooser(viewerModule, param, propertyPanel);
				if (paramChooser != null)
				{
					paramChoosers.put(param.getPrompt(), paramChooser);
					paramChooser.updateValue();
				}
			}
		}
		
		for (Map.Entry<String, ParamChooser> entry : paramChoosers.entrySet())
        {
	        Field field = (Field) entry.getValue();
	        field.setLabel(entry.getKey());
	        
	        propertyPanel.add(field, new AnchorLayoutData("100%"));
        }
		
		return propertyPanel;
	}
	
	public static String getLastInputValue(String paramName)
	{
		return sLastInputValues.get(paramName);
	}

	public static void setLastInputValue(String paramName, String paramValue)
	{
		sLastInputValues.put(paramName, paramValue);
	}
}
