/*
 * Copyright (c) 2008 appspy.org, Contributors.
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
 * Unless required by applicable law or agreed to in writing, software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions 
 * and limitations under the License. */

package org.appspy.viewer.client.paramchooser;

import java.util.Collection;

import org.appspy.viewer.client.ViewerModule;
import org.appspy.viewer.client.action.AbstractAction;
import org.appspy.viewer.client.form.ReportParamForm;

import com.google.gwt.core.client.GWT;
import com.gwtext.client.core.EventObject;
import com.gwtext.client.widgets.Button;
import com.gwtext.client.widgets.Panel;
import com.gwtext.client.widgets.Window;
import com.gwtext.client.widgets.event.ButtonListenerAdapter;
import com.gwtext.client.widgets.layout.FitLayout;

/**
 * @author Olivier HEDIN / olivier@appspy.org
 */
public class ParamWindow extends Window {
	
	protected Panel mPropertyPanel = null;
	protected AbstractAction mAction = null;

	public ParamWindow(ViewerModule viewerModule, final Collection<ReportParamForm> params, final AbstractAction action)
	{
		super();
		
		mAction = action;
		
		setTitle("Parameter values");
		setFrame(true);
		setLayout(new FitLayout());
		setSize(400, 300);
		
		mPropertyPanel = ParamPropertyGridPanelFactory.getPropertyGridPanel(viewerModule, params);
		
		add(mPropertyPanel);
		
		Button okButton = new Button("OK");
		okButton.addListener(new ButtonListenerAdapter(){

			@Override
			public void onClick(Button button, EventObject e) {
				try
				{
					for (ReportParamForm param : params)
                    {
	                    ParamPropertyGridPanelFactory.setLastInputValue(param.getName(), param.getDefault());
                    }
					executeAction();
					destroy();
				}
				catch (Exception ex)
				{
					GWT.log(ex.getMessage(), ex);
				}
			}});
		addButton(okButton);
		
	}
	
	public void executeAction()
	{
		mAction.execute();
	}
}
