/*
 * Copyright (c) 2008 appspy.org, Contributors.
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
 * Unless required by applicable law or agreed to in writing, software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions 
 * and limitations under the License. */

package org.appspy.viewer.client.paramchooser.old;

import java.util.Collection;
import java.util.HashMap;

import org.appspy.viewer.client.ViewerModule;
import org.appspy.viewer.client.form.ReportParamForm;

import com.gwtext.client.widgets.form.Field;
import com.gwtext.client.widgets.form.event.FieldListenerAdapter;
import com.gwtext.client.widgets.grid.GridEditor;
import com.gwtext.client.widgets.grid.PropertyGridPanel;
import com.gwtext.client.widgets.grid.event.PropertyGridPanelListener;

/**
 * @author Olivier HEDIN / olivier@appspy.org
 */
public class ParamPropertyGridPanelFactory {
	
	protected static HashMap<String, String> sLastInputValues = new HashMap<String, String>();
	
	public static PropertyGridPanel getPropertyGridPanel(ViewerModule viewerModule, Collection<ReportParamForm> params)
	{
		final PropertyGridPanel propertyGridPanel = new PropertyGridPanel();
		propertyGridPanel.setAutoScroll(true);
		propertyGridPanel.setSorted(true);
		
		final HashMap<String, String> source = new HashMap<String, String>();
		final HashMap<String, GridEditor> editors = new HashMap<String, GridEditor>();
		final HashMap<String, Field> fields = new HashMap<String, Field>();
		final HashMap<String, ParamChooser> paramChoosers = new HashMap<String, ParamChooser>();
		for (ReportParamForm param : params) {
			if (param.getDefault() == null || "".equals(param.getDefault()))
			{
				if (sLastInputValues.containsKey(param.getName()))
				{
					param.setDefault(sLastInputValues.get(param.getName()));
				}
			}
			ParamChooser paramChooser = ParamChooserFactory.getParamChooser(viewerModule, param, propertyGridPanel, source);
			if (paramChooser != null)
			{
				source.put(param.getName(), param.getDefault());
				Field field = (Field) paramChooser;
				field.addListener(new FieldListenerAdapter(){

                    @Override
                    public void onBlur(Field field)
                    {
                		propertyGridPanel.setSource(source);
                    }});
				editors.put(param.getName(), new GridEditor(field));
				fields.put(param.getName(), field);
				paramChoosers.put(param.getName(), paramChooser);
				paramChooser.updateValue();
			}
		}
		propertyGridPanel.setSource(source);
		propertyGridPanel.setCustomEditors(editors);
		propertyGridPanel.addPropertyGridPanelListener(new PropertyGridPanelListener(){

			public boolean doBeforePropertyChange(PropertyGridPanel source, String recordID, Object value, Object oldValue) {
				return true;
			}

			public void onPropertyChange(PropertyGridPanel propertyGridPanel, String recordID, Object value, Object oldValue) {
//				ParamChooser chooser = paramChoosers.get(recordID);
//				chooser.getParam().setDefault(value.toString());
//				chooser.updateValue();
			}});
		
		
		return propertyGridPanel;
	}
	
	public static String getLastInputValue(String paramName)
	{
		return sLastInputValues.get(paramName);
	}

	public static void setLastInputValue(String paramName, String paramValue)
	{
		sLastInputValues.put(paramName, paramValue);
	}
}
