/*
 * Copyright (c) 2008 appspy.org, Contributors.
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
 * Unless required by applicable law or agreed to in writing, software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions 
 * and limitations under the License. */

package org.appspy.viewer.client.paramchooser.old;

import java.util.Map;

import org.appspy.viewer.client.ViewerModule;
import org.appspy.viewer.client.form.ReportParamForm;

import com.google.gwt.core.client.GWT;
import com.gwtext.client.core.EventObject;
import com.gwtext.client.widgets.Button;
import com.gwtext.client.widgets.Component;
import com.gwtext.client.widgets.Window;
import com.gwtext.client.widgets.event.ButtonListenerAdapter;
import com.gwtext.client.widgets.form.Field;
import com.gwtext.client.widgets.form.FormPanel;
import com.gwtext.client.widgets.form.TextField;
import com.gwtext.client.widgets.form.event.TextFieldListenerAdapter;
import com.gwtext.client.widgets.grid.PropertyGridPanel;
import com.gwtext.client.widgets.layout.AnchorLayoutData;
import com.gwtext.client.widgets.layout.FitLayout;

/**
 * @author Olivier HEDIN / olivier@appspy.org
 */
public class TextChooser extends TriggerFieldParamChooser {
	
	protected String mParamValue = null;
	protected String mParamDisplayValue = null;
	
	public TextChooser(ViewerModule viewerModule, ReportParamForm param, PropertyGridPanel propertyGridPanel, Map<String, String> propertyGridMap)
	{
		super(viewerModule, param, propertyGridPanel, propertyGridMap);
	}
	
	/* (non-Javadoc)
	 * @see com.gwtext.client.widgets.form.TriggerField#onTriggerClick(com.gwtext.client.core.EventObject)
	 */
	@Override
	protected void onTriggerClick(EventObject event) {
		try
		{
			final Window w = new Window(mParam.getPrompt());
			w.setFrame(true);
			w.setModal(true);
			w.setLayout(new FitLayout());
			w.setSize(300, 150);
			
			w.add(getChooserComponent());
			
			w.addButton(new Button("OK", new ButtonListenerAdapter(){

				@Override
				public void onClick(Button button, EventObject e) {
					mParam.setDefault(mParamValue);
					mPropertyGridMap.put(mParam.getName(), mParamDisplayValue);
					mPropertyGridPanel.setSource(mPropertyGridMap);
					
					w.destroy();
				}}));
			
			w.addButton(new Button("Cancel", new ButtonListenerAdapter(){

				/* (non-Javadoc)
				 * @see com.gwtext.client.widgets.event.ButtonListenerAdapter#onClick(com.gwtext.client.widgets.Button, com.gwtext.client.core.EventObject)
				 */
				@Override
				public void onClick(Button button, EventObject e) {
					w.destroy();
				}}));
			
			w.show();
		}
		catch (Exception ex)
		{
			GWT.log(ex.getMessage(), ex);
		}
	}
	
	/* (non-Javadoc)
	 * @see org.appspy.admin.client.paramchooser.ParamChooser#getChooserComponent()
	 */
	public Component getChooserComponent() {
		
		final FormPanel formPanel = new FormPanel();
		formPanel.setFrame(true);
		
		final TextField textField = new TextField(mParam.getPrompt());
		textField.setValue(mParam.getDefault());
		formPanel.add(textField, new AnchorLayoutData("100%"));
		
		textField.addListener(new TextFieldListenerAdapter(){

			/* (non-Javadoc)
             * @see com.gwtext.client.widgets.form.event.FieldListenerAdapter#onChange(com.gwtext.client.widgets.form.Field, java.lang.Object, java.lang.Object)
             */
            @Override
            public void onChange(Field field, Object newVal, Object oldVal)
            {
				mParamDisplayValue = newVal.toString();
				mParamValue = newVal.toString();
			}});
		
		return formPanel;
	}

	public void updateValue() {
		mPropertyGridMap.put(mParam.getName(), mParam.getDefault());
		mPropertyGridPanel.setSource(mPropertyGridMap);
	}

	/**
	 * @return the paramValue
	 */
	public String getParamValue() {
		return mParamValue;
	}

	/**
	 * @param paramValue the paramValue to set
	 */
	public void setParamValue(String paramValue) {
		mParamValue = paramValue;
	}

	/**
	 * @return the paramDisplayValue
	 */
	public String getParamDisplayValue() {
		return mParamDisplayValue;
	}

	/**
	 * @param paramDisplayValue the paramDisplayValue to set
	 */
	public void setParamDisplayValue(String paramDisplayValue) {
		mParamDisplayValue = paramDisplayValue;
	}
}
