/*
 * Copyright (c) 2008 appspy.org, Contributors.
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
 * Unless required by applicable law or agreed to in writing, software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions 
 * and limitations under the License. */

package org.appspy.viewer.client.paramchooser.old;

import java.util.Collection;
import java.util.Map;
import java.util.TreeMap;

import org.appspy.viewer.client.ViewerModule;
import org.appspy.viewer.client.form.ApplicationForm;
import org.appspy.viewer.client.form.ApplicationVersionForm;
import org.appspy.viewer.client.form.ReportParamForm;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.gwtext.client.core.EventObject;
import com.gwtext.client.widgets.Button;
import com.gwtext.client.widgets.Component;
import com.gwtext.client.widgets.Window;
import com.gwtext.client.widgets.event.ButtonListenerAdapter;
import com.gwtext.client.widgets.grid.PropertyGridPanel;
import com.gwtext.client.widgets.layout.FitLayout;
import com.gwtext.client.widgets.tree.TreeNode;
import com.gwtext.client.widgets.tree.TreePanel;
import com.gwtext.client.widgets.tree.event.TreePanelListenerAdapter;

/**
 * @author Olivier HEDIN / olivier@appspy.org
 */
public class VersionChooser extends TriggerFieldParamChooser {
	
	protected String mParamValue = null;
	protected String mParamDisplayValue = null;
	
	public VersionChooser(ViewerModule viewerModule, ReportParamForm param, PropertyGridPanel propertyGridPanel, Map<String, String> propertyGridMap)
	{
		super(viewerModule, param, propertyGridPanel, propertyGridMap);
	}
	
	/* (non-Javadoc)
	 * @see com.gwtext.client.widgets.form.TriggerField#onTriggerClick(com.gwtext.client.core.EventObject)
	 */
	@Override
	protected void onTriggerClick(EventObject event) {
		try
		{
			final Window w = new Window(mParam.getPrompt());
			w.setFrame(true);
			w.setModal(true);
			w.setLayout(new FitLayout());
			w.setSize(400, 300);
			
			w.add(getChooserComponent());
			
			w.addButton(new Button("OK", new ButtonListenerAdapter(){

				@Override
				public void onClick(Button button, EventObject e) {
					mParam.setDefault(mParamValue);
					mPropertyGridMap.put(mParam.getName(), mParamDisplayValue);
					mPropertyGridPanel.setSource(mPropertyGridMap);
					
					w.destroy();
				}}));
			
			w.addButton(new Button("Cancel", new ButtonListenerAdapter(){

				/* (non-Javadoc)
				 * @see com.gwtext.client.widgets.event.ButtonListenerAdapter#onClick(com.gwtext.client.widgets.Button, com.gwtext.client.core.EventObject)
				 */
				@Override
				public void onClick(Button button, EventObject e) {
					w.destroy();
				}}));
			
			w.show();
		}
		catch (Exception ex)
		{
			GWT.log(ex.getMessage(), ex);
		}
	}
	
	/* (non-Javadoc)
	 * @see org.appspy.admin.client.paramchooser.ParamChooser#getChooserComponent()
	 */
	public Component getChooserComponent() {
		
		final TreePanel treePanel = new TreePanel();
		
		treePanel.setTitle("Application versions");
		treePanel.setAutoScroll(true);
		treePanel.setBorder(false);
		treePanel.setIconCls("settings-menu-icon");
		treePanel.setEnableDD(false);
		TreeNode root = new TreeNode("Application versions");
		root.setExpanded(true);
		treePanel.setRootNode(root);
		treePanel.setRootVisible(false);
		
		getViewerModule().getViewerModuleService().getVersions(new AsyncCallback<Collection<ApplicationVersionForm>>(){

			public void onFailure(Throwable caught) {
			}

			public void onSuccess(Collection<ApplicationVersionForm> result) {

				TreeMap<String, ApplicationForm> appMap = new TreeMap<String, ApplicationForm>();
				TreeMap<String, ApplicationVersionForm> versionMap = new TreeMap<String, ApplicationVersionForm>();
				for (ApplicationVersionForm version : result) {
					appMap.put(version.getApplication().getName(), version.getApplication());
					versionMap.put(version.getApplication().getName()+version.getName(), version);
				}
				for (ApplicationForm app : appMap.values()) {
					final TreeNode appTreeNode = new TreeNode(app.getName());
					appTreeNode.setUserObject(app);
					appTreeNode.setIconCls("settings-menu-icon");
					treePanel.getRootNode().appendChild(appTreeNode);
					for (ApplicationVersionForm version : versionMap.values()) {
						if (version.getApplication() == app)
						{
							final TreeNode versionTreeNode = new TreeNode(version.getName());
							versionTreeNode.setUserObject(version);
							versionTreeNode.setIconCls("settings-menu-icon");
							appTreeNode.appendChild(versionTreeNode);
							if (version.getId().toString().equals(mParam.getDefault()))
							{
								versionTreeNode.ensureVisible();
								versionTreeNode.select();
							}
						}
					}
				}
			}});
		
		treePanel.addListener(new TreePanelListenerAdapter(){

			@Override
			public void onClick(TreeNode node, EventObject e) {
				try
				{
					ApplicationVersionForm version = (ApplicationVersionForm) node.getUserObject();
					mParamDisplayValue = version.getName();
					mParamValue = version.getId().toString();
				}
				catch (Exception ex)
				{
					GWT.log(ex.getMessage(), ex);
				}
			}});
		
		return treePanel;
	}

	public void updateValue() {
		getViewerModule().getViewerModuleService().getVersions(new AsyncCallback<Collection<ApplicationVersionForm>>(){

			public void onFailure(Throwable caught) {
				GWT.log(caught.getMessage(), caught);
			}

			public void onSuccess(Collection<ApplicationVersionForm> result) {
				for (ApplicationVersionForm version : result) {
					if (version.getId().toString().equals(mParam.getDefault()))
					{
						mPropertyGridMap.put(mParam.getName(), version.getName());
						mPropertyGridPanel.setSource(mPropertyGridMap);
						return;
					}
				}
			}});
	}

	/**
	 * @return the paramValue
	 */
	public String getParamValue() {
		return mParamValue;
	}

	/**
	 * @param paramValue the paramValue to set
	 */
	public void setParamValue(String paramValue) {
		mParamValue = paramValue;
	}

	/**
	 * @return the paramDisplayValue
	 */
	public String getParamDisplayValue() {
		return mParamDisplayValue;
	}

	/**
	 * @param paramDisplayValue the paramDisplayValue to set
	 */
	public void setParamDisplayValue(String paramDisplayValue) {
		mParamDisplayValue = paramDisplayValue;
	}
}
