/*
 * Copyright (c) 2008 appspy.org, Contributors.
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
 * Unless required by applicable law or agreed to in writing, software distributed under the 
 * License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language governing permissions 
 * and limitations under the License. */

package org.appspy.viewer.client.reports;

import java.util.Collection;
import java.util.HashMap;

import org.appspy.viewer.client.AbstractTreePanel;
import org.appspy.viewer.client.ViewerModule;
import org.appspy.viewer.client.form.ReportCategoryForm;
import org.appspy.viewer.client.form.ReportForm;
import org.appspy.viewer.client.reports.actions.OpenReportAction;

import com.google.gwt.user.client.rpc.AsyncCallback;
import com.gwtext.client.core.EventObject;
import com.gwtext.client.data.Node;
import com.gwtext.client.widgets.MessageBox;
import com.gwtext.client.widgets.tree.TreeNode;
import com.gwtext.client.widgets.tree.event.TreeNodeListenerAdapter;

/**
 * @author Olivier HEDIN / olivier@appspy.org
 */
public class ReportsMenuTreePanel extends AbstractTreePanel {
	
	protected TreeNode mRoot = null;

	public ReportsMenuTreePanel(ViewerModule viewerModule) {

		setViewerModule(viewerModule);
		mRoot = new TreeNode("Reports");
		mRoot.setExpanded(true);

		setTitle("Reports");
		setIconCls("reports-menu-icon");
		setWidth(200);
		setHeight(400);
		setEnableDD(true);
		setRootNode(mRoot);
		setRootVisible(false);
		
		refreshDatas();
	}
	
	public void refreshDatas()
	{
		getViewerModule().getViewerModuleService().getReports(new AsyncCallback<Collection<ReportForm>>(){
			
			public void onFailure(Throwable caught) {
				MessageBox.alert("Error", caught.getLocalizedMessage());
			}
			
			public void onSuccess(Collection<ReportForm> result) {
				HashMap<String, ReportCategoryForm> cats = new HashMap<String, ReportCategoryForm>();
				for (ReportForm report : result) {
					if (report.getHidden() == null || !report.getHidden())
					{
						cats.put(report.getCategory().getName(), report.getCategory());
					}
				}
				for (ReportCategoryForm cat : cats.values()) {
					TreeNode catTreeNode = new TreeNode(cat.getName());
					catTreeNode.setIconCls("reports-menu-icon");
					mRoot.appendChild(catTreeNode);
					for (final ReportForm report : result) {
						if (report.getHidden() != null && report.getHidden())
						{
							continue;
						}
						if (report.getCategory().getId() == cat.getId())
						{
							TreeNode reportTreeNode = new TreeNode(report.getName());
							reportTreeNode.setIconCls("reports-menu-icon");
							catTreeNode.appendChild(reportTreeNode);
							reportTreeNode.addListener(new TreeNodeListenerAdapter() {
								
								@Override
								public void onClick(Node node, EventObject e) {
									
									AsyncCallback<ReportForm> callback = new AsyncCallback<ReportForm>(){
										
										public void onFailure(Throwable caught) {
											MessageBox.alert("Error", caught.getLocalizedMessage());
										}
										
										public void onSuccess(ReportForm result) {
											new OpenReportAction(getViewerModule(), result, true).execute();
										}};
										
										getViewerModule().getViewerModuleService().getReport(report.getId(), callback);
								}});
						}
					}
				}
			}});
	}
}
